@@ -1,13 +1,14 @@
+from attr import attrib, attrs
 from copy import deepcopy
 
 from jsonschema import _legacy_validators, _utils, _validators
-from jsonschema.validators import create
+from jsonschema.validators import create, Draft202012Validator, extend
 
 from openapi_schema_validator import _types as oas_types
 from openapi_schema_validator import _validators as oas_validators
+from openapi_schema_validator._types import oas31_type_checker
 
-
-BaseOAS30Validator = create(
+OAS30Validator = create(
     meta_schema=_utils.load_schema("draft4"),
     validators={
         u"multipleOf": _validators.multipleOf,
@@ -26,9 +27,9 @@ BaseOAS30Validator = create(
         u"enum": _validators.enum,
         # adjusted to OAS
         u"type": oas_validators.type,
-        u"allOf": _validators.allOf,
-        u"oneOf": _validators.oneOf,
-        u"anyOf": _validators.anyOf,
+        u"allOf": oas_validators.allOf,
+        u"oneOf": oas_validators.oneOf,
+        u"anyOf": oas_validators.anyOf,
         u"not": _validators.not_,
         u"items": oas_validators.items,
         u"properties": _validators.properties,
@@ -53,25 +54,41 @@ BaseOAS30Validator = create(
     # See https://github.com/p1c2u/openapi-schema-validator/pull/12
     # version="oas30",
     id_of=lambda schema: schema.get(u"id", ""),
+    applicable_validators=oas_validators.include_nullable_validator,
+)
+
+OAS31Validator = extend(
+    Draft202012Validator,
+    {
+        # adjusted to OAS
+        u"allOf": oas_validators.allOf,
+        u"oneOf": oas_validators.oneOf,
+        u"anyOf": oas_validators.anyOf,
+        u"description": oas_validators.not_implemented,
+        u"format": oas_validators.format,
+        # fixed OAS fields
+        u"discriminator": oas_validators.not_implemented,
+        u"xml": oas_validators.not_implemented,
+        u"externalDocs": oas_validators.not_implemented,
+        u"example": oas_validators.not_implemented,
+    },
+    type_checker=oas31_type_checker,
 )
 
 
-class OAS30Validator(BaseOAS30Validator):
+def _patch_validator_with_read_write_context(cls):
+    """Adds read/write context to jsonschema validator class"""
+    # subclassing validator classes is not intended to
+    # be part of their public API and will raise error
+    # See https://github.com/p1c2u/openapi-schema-validator/issues/48
+    original_init = cls.__init__
 
     def __init__(self, *args, **kwargs):
-        self.read = kwargs.pop('read', None)
-        self.write = kwargs.pop('write', None)
-        super(OAS30Validator, self).__init__(*args, **kwargs)
+        self.read = kwargs.pop("read", None)
+        self.write = kwargs.pop("write", None)
+        original_init(self, *args, **kwargs)
 
-    def iter_errors(self, instance, _schema=None):
-        if _schema is None:
-            # creates a copy by value from schema to prevent mutation
-            _schema = deepcopy(self.schema)
+    cls.__init__ = __init__
 
-        # append defaults to trigger validator (i.e. nullable)
-        if 'nullable' not in _schema:
-            _schema.update({
-                'nullable': False,
-            })
 
-        return super(OAS30Validator, self).iter_errors(instance, _schema)
+_patch_validator_with_read_write_context(OAS30Validator)
